%pointer
%e 100
%p 200
%n 40
%k 50
%a 100
%o 500


%{
/*
 * Copyright 2006, Thomas Maier-Komor
 * LICENSE: GPLv2, see file COPYING for details
 */

#include "log.h"
#include "tokens.h"
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#ifdef DEBUGLEX
#define debuglex1 debug
#define debuglex2 debug
#else
#define debuglex1(a)
#define debuglex2(a,b)
#endif

char *Buffer;
size_t Fill;

%}

WHITESP		[ \t]+
COMMENT		^#[^\n]*\n
SEQUPT		^%%\n
EQUOTE		\\[^\n]
DQUOTED		\"([^"\n\\]|{EQUOTE})*\"
SQUOTED		"'"([^'\n\\]|{EQUOTE})*"'"
ESPACE		"\\ "
UQUOTED		[^ \t\n'"<>]([^ \t\n]|{ESPACE})*
RDIN		"<"
RDOUT		">"
RDAPP		">>"
RDOE		">&"
RDAOE		">>&"

%%

{WHITESP}	{
	debuglex1("W ");
	}

{UQUOTED}	{
	char *at = yytext, *esc;
	Fill = yyleng;
	Buffer = yytext;
	while ((esc = strchr(at,'\\'))) {
		at = esc + 1;
		if ((*at == ' ') || (*at == '\t')) {
			size_t l = Fill - (at - Buffer) + 1;
			memmove(esc, at, l);
			--Fill;
		}
	}
	Buffer[Fill] = 0;
	debuglex2("UQ(%s) ",Buffer);
	return UQUOTED;
	}

{DQUOTED}	{
	char *at = yytext + 1, *esc;
	Fill = yyleng - 2;
	Buffer = at;
	while ((esc = strchr(at,'\\'))) {
		at = esc + 1;
		if (*at == '"') {
			size_t l = Fill - (at - Buffer) + 1;
			memmove(esc, at, l);
			--Fill;
		}
	}
	Buffer[Fill] = 0;
	debuglex2("DQ(%s) ",Buffer);
	return DQUOTED;
	}

{SQUOTED}	{
	char *at = yytext + 1, *esc;
	Fill = yyleng - 2;
	Buffer = at;
	while ((esc = strchr(at,'\\'))) {
		at = esc + 1;
		if (*at == '\'') {
			size_t l = Fill - (at - Buffer) + 1;
			memmove(esc, at, l);
			--Fill;
		}
	}
	Buffer[Fill] = 0;
	debuglex2("S(%s) ",Buffer);
	return SQUOTED;
	}

{RDIN}		{
	debuglex1("I ");
	return INPUT;
	}

{RDAPP}		{
	debuglex1("a ");
	return APPEND;
	}

{RDOUT}		{
	debuglex1("O ");
	return OUTPUT;
	}

{RDOE}		{
	debuglex1("E ");
	return ERRLOG;
	}

{RDAOE}		{
	debuglex1("A ");
	return APPERR;
	}

{SEQUPT}	{
	debuglex1("P ");
	return SEQUPT;
	}

{COMMENT}	{
	}

\n		{
	debuglex1("E\n");
	return EOL;
	}

.		{
	error("unexpected charakter in input: %s\n", yytext);
	return ERR;
	}

%%

int yywrap(void)
{
	return 1;
}


#ifdef LEXDEBUG
void sprint(char *s)
{
        dbug(s);
}

void allprint(char c)
{
        dbug("%c",c);
}
#endif

